# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import (
    Kind106,
    Kind107,
    Kind108,
    Kind109,
    Kind110,
    Kind111,
    Kind112,
    Kind113,
    Kind114,
    Kind115,
    Kind116,
    Kind117,
    Kind118,
    Kind119,
)
from ...apimachinery.pkg.apis.meta import v1
from ...apimachinery.pkg.util import intstr
from ..core import v1 as v1_1


class AllowedCSIDriver(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: str = Field(..., description="Name is the registered name of the CSI driver")


class AllowedFlexVolume(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    driver: str = Field(..., description="driver is the name of the Flexvolume driver.")


class AllowedHostPath(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    path_prefix: Optional[str] = Field(
        None,
        alias="pathPrefix",
        description="pathPrefix is the path prefix that the host volume must match. It does not support `*`. Trailing slashes are trimmed when validating the path prefix with a host path.\n\nExamples: `/foo` would allow `/foo`, `/foo/` and `/foo/bar` `/foo` would not allow `/food` or `/etc/foo`",
    )
    read_only: Optional[bool] = Field(
        None,
        alias="readOnly",
        description="when set to true, will allow host volumes matching the pathPrefix only if all volume mounts are readOnly.",
    )


class HostPortRange(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max: int = Field(..., description="max is the end of the range, inclusive.")
    min: int = Field(..., description="min is the start of the range, inclusive.")


class IDRange(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max: int = Field(..., description="max is the end of the range, inclusive.")
    min: int = Field(..., description="min is the start of the range, inclusive.")


class IPBlock(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    cidr: str = Field(
        ...,
        description='CIDR is a string representing the IP Block Valid examples are "192.168.1.1/24"',
    )
    except_: Optional[List[str]] = Field(
        None,
        alias="except",
        description='Except is a slice of CIDRs that should not be included within an IP Block Valid examples are "192.168.1.1/24" Except values will be rejected if they are outside the CIDR range',
    )


class IngressStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    load_balancer: Optional[v1_1.LoadBalancerStatus] = Field(
        None,
        alias="loadBalancer",
        description="LoadBalancer contains the current status of the load-balancer.",
    )


class IngressTLS(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    hosts: Optional[List[str]] = Field(
        None,
        description="Hosts are a list of hosts included in the TLS certificate. The values in this list must match the name/s used in the tlsSecret. Defaults to the wildcard host setting for the loadbalancer controller fulfilling this Ingress, if left unspecified.",
    )
    secret_name: Optional[str] = Field(
        None,
        alias="secretName",
        description='SecretName is the name of the secret used to terminate SSL traffic on 443. Field is left optional to allow SSL routing based on SNI hostname alone. If the SNI host in a listener conflicts with the "Host" header field used by an IngressRule, the SNI host is used for termination and value of the Host header is used for routing.',
    )


class RollbackConfig(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    revision: Optional[int] = Field(
        None,
        description="The revision to rollback to. If set to 0, rollback to the last revision.",
    )


class RunAsGroupStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of gids that may be used. If you would like to force a single gid then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: str = Field(
        ...,
        description="rule is the strategy that will dictate the allowable RunAsGroup values that may be set.",
    )


class RunAsUserStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of uids that may be used. If you would like to force a single uid then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: str = Field(
        ...,
        description="rule is the strategy that will dictate the allowable RunAsUser values that may be set.",
    )


class RuntimeClassStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    allowed_runtime_class_names: List[str] = Field(
        ...,
        alias="allowedRuntimeClassNames",
        description='allowedRuntimeClassNames is a whitelist of RuntimeClass names that may be specified on a pod. A value of "*" means that any RuntimeClass name is allowed, and must be the only item in the list. An empty list requires the RuntimeClassName field to be unset.',
    )
    default_runtime_class_name: Optional[str] = Field(
        None,
        alias="defaultRuntimeClassName",
        description="defaultRuntimeClassName is the default RuntimeClassName to set on the pod. The default MUST be allowed by the allowedRuntimeClassNames list. A value of nil does not mutate the Pod.",
    )


class SELinuxStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rule: str = Field(
        ...,
        description="rule is the strategy that will dictate the allowable labels that may be set.",
    )
    se_linux_options: Optional[v1_1.SELinuxOptions] = Field(
        None,
        alias="seLinuxOptions",
        description="seLinuxOptions required to run as; required for MustRunAs More info: https://kubernetes.io/docs/tasks/configure-pod-container/security-context/",
    )


class ScaleSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    replicas: Optional[int] = Field(
        None, description="desired number of instances for the scaled object."
    )


class ScaleStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    replicas: int = Field(
        ..., description="actual number of observed instances of the scaled object."
    )
    selector: Optional[Dict[str, Any]] = Field(
        None,
        description="label query over pods that should match the replicas count. More info: http://kubernetes.io/docs/user-guide/labels#label-selectors",
    )
    target_selector: Optional[str] = Field(
        None,
        alias="targetSelector",
        description="label selector for pods that should match the replicas count. This is a serializated version of both map-based and more expressive set-based selectors. This is done to avoid introspection in the clients. The string will be in the same format as the query-param syntax. If the target type only supports map-based selectors, both this field and map-based selector field are populated. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )


class SupplementalGroupsStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of supplemental groups.  If you would like to force a single supplemental group then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: Optional[str] = Field(
        None,
        description="rule is the strategy that will dictate what supplemental groups is used in the SecurityContext.",
    )


class DaemonSetCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of DaemonSet condition.")


class DaemonSetStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    collision_count: Optional[int] = Field(
        None,
        alias="collisionCount",
        description="Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
    )
    conditions: Optional[List[DaemonSetCondition]] = Field(
        None,
        description="Represents the latest available observations of a DaemonSet's current state.",
    )
    current_number_scheduled: int = Field(
        ...,
        alias="currentNumberScheduled",
        description="The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
    )
    desired_number_scheduled: int = Field(
        ...,
        alias="desiredNumberScheduled",
        description="The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
    )
    number_available: Optional[int] = Field(
        None,
        alias="numberAvailable",
        description="The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
    )
    number_misscheduled: int = Field(
        ...,
        alias="numberMisscheduled",
        description="The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
    )
    number_ready: int = Field(
        ...,
        alias="numberReady",
        description="The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and ready.",
    )
    number_unavailable: Optional[int] = Field(
        None,
        alias="numberUnavailable",
        description="The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="The most recent generation observed by the daemon set controller.",
    )
    updated_number_scheduled: Optional[int] = Field(
        None,
        alias="updatedNumberScheduled",
        description="The total number of nodes that are running updated daemon pod",
    )


class DeploymentCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transitioned from one status to another.",
    )
    last_update_time: Optional[v1.Time] = Field(
        None,
        alias="lastUpdateTime",
        description="The last time this condition was updated.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of deployment condition.")


class DeploymentRollback(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind110] = Field(
        "DeploymentRollback",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    name: str = Field(
        ..., description="Required: This must match the Name of a deployment."
    )
    rollback_to: RollbackConfig = Field(
        ..., alias="rollbackTo", description="The config of this deployment rollback."
    )
    updated_annotations: Optional[Dict[str, Any]] = Field(
        None,
        alias="updatedAnnotations",
        description="The annotations to be updated to a deployment",
    )


class DeploymentStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    available_replicas: Optional[int] = Field(
        None,
        alias="availableReplicas",
        description="Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
    )
    collision_count: Optional[int] = Field(
        None,
        alias="collisionCount",
        description="Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
    )
    conditions: Optional[List[DeploymentCondition]] = Field(
        None,
        description="Represents the latest available observations of a deployment's current state.",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="The generation observed by the deployment controller.",
    )
    ready_replicas: Optional[int] = Field(
        None,
        alias="readyReplicas",
        description="Total number of ready pods targeted by this deployment.",
    )
    replicas: Optional[int] = Field(
        None,
        description="Total number of non-terminated pods targeted by this deployment (their labels match the selector).",
    )
    unavailable_replicas: Optional[int] = Field(
        None,
        alias="unavailableReplicas",
        description="Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
    )
    updated_replicas: Optional[int] = Field(
        None,
        alias="updatedReplicas",
        description="Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
    )


class FSGroupStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of fs groups.  If you would like to force a single fs group then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: Optional[str] = Field(
        None,
        description="rule is the strategy that will dictate what FSGroup is used in the SecurityContext.",
    )


class IngressBackend(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    service_name: str = Field(
        ...,
        alias="serviceName",
        description="Specifies the name of the referenced service.",
    )
    service_port: Union[int, str] = Field(
        ...,
        alias="servicePort",
        description="Specifies the port of the referenced service.",
    )


class NetworkPolicyPort(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    port: Optional[Union[int, str]] = Field(
        None,
        description="If specified, the port on the given protocol.  This can either be a numerical or named port on a pod.  If this field is not provided, this matches all port names and numbers. If present, only traffic on the specified protocol AND port will be matched.",
    )
    protocol: Optional[str] = Field(
        None,
        description="Optional.  The protocol (TCP, UDP, or SCTP) which traffic must match. If not specified, this field defaults to TCP.",
    )


class PodSecurityPolicySpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    allow_privilege_escalation: Optional[bool] = Field(
        None,
        alias="allowPrivilegeEscalation",
        description="allowPrivilegeEscalation determines if a pod can request to allow privilege escalation. If unspecified, defaults to true.",
    )
    allowed_csi_drivers: Optional[List[AllowedCSIDriver]] = Field(
        None,
        alias="allowedCSIDrivers",
        description="AllowedCSIDrivers is a whitelist of inline CSI drivers that must be explicitly set to be embedded within a pod spec. An empty value indicates that any CSI driver can be used for inline ephemeral volumes.",
    )
    allowed_capabilities: Optional[List[str]] = Field(
        None,
        alias="allowedCapabilities",
        description="allowedCapabilities is a list of capabilities that can be requested to add to the container. Capabilities in this field may be added at the pod author's discretion. You must not list a capability in both allowedCapabilities and requiredDropCapabilities.",
    )
    allowed_flex_volumes: Optional[List[AllowedFlexVolume]] = Field(
        None,
        alias="allowedFlexVolumes",
        description='allowedFlexVolumes is a whitelist of allowed Flexvolumes.  Empty or nil indicates that all Flexvolumes may be used.  This parameter is effective only when the usage of the Flexvolumes is allowed in the "volumes" field.',
    )
    allowed_host_paths: Optional[List[AllowedHostPath]] = Field(
        None,
        alias="allowedHostPaths",
        description="allowedHostPaths is a white list of allowed host paths. Empty indicates that all host paths may be used.",
    )
    allowed_proc_mount_types: Optional[List[str]] = Field(
        None,
        alias="allowedProcMountTypes",
        description="AllowedProcMountTypes is a whitelist of allowed ProcMountTypes. Empty or nil indicates that only the DefaultProcMountType may be used. This requires the ProcMountType feature flag to be enabled.",
    )
    allowed_unsafe_sysctls: Optional[List[str]] = Field(
        None,
        alias="allowedUnsafeSysctls",
        description='allowedUnsafeSysctls is a list of explicitly allowed unsafe sysctls, defaults to none. Each entry is either a plain sysctl name or ends in "*" in which case it is considered as a prefix of allowed sysctls. Single * means all unsafe sysctls are allowed. Kubelet has to whitelist all allowed unsafe sysctls explicitly to avoid rejection.\n\nExamples: e.g. "foo/*" allows "foo/bar", "foo/baz", etc. e.g. "foo.*" allows "foo.bar", "foo.baz", etc.',
    )
    default_add_capabilities: Optional[List[str]] = Field(
        None,
        alias="defaultAddCapabilities",
        description="defaultAddCapabilities is the default set of capabilities that will be added to the container unless the pod spec specifically drops the capability.  You may not list a capability in both defaultAddCapabilities and requiredDropCapabilities. Capabilities added here are implicitly allowed, and need not be included in the allowedCapabilities list.",
    )
    default_allow_privilege_escalation: Optional[bool] = Field(
        None,
        alias="defaultAllowPrivilegeEscalation",
        description="defaultAllowPrivilegeEscalation controls the default setting for whether a process can gain more privileges than its parent process.",
    )
    forbidden_sysctls: Optional[List[str]] = Field(
        None,
        alias="forbiddenSysctls",
        description='forbiddenSysctls is a list of explicitly forbidden sysctls, defaults to none. Each entry is either a plain sysctl name or ends in "*" in which case it is considered as a prefix of forbidden sysctls. Single * means all sysctls are forbidden.\n\nExamples: e.g. "foo/*" forbids "foo/bar", "foo/baz", etc. e.g. "foo.*" forbids "foo.bar", "foo.baz", etc.',
    )
    fs_group: FSGroupStrategyOptions = Field(
        ...,
        alias="fsGroup",
        description="fsGroup is the strategy that will dictate what fs group is used by the SecurityContext.",
    )
    host_ipc: Optional[bool] = Field(
        None,
        alias="hostIPC",
        description="hostIPC determines if the policy allows the use of HostIPC in the pod spec.",
    )
    host_network: Optional[bool] = Field(
        None,
        alias="hostNetwork",
        description="hostNetwork determines if the policy allows the use of HostNetwork in the pod spec.",
    )
    host_pid: Optional[bool] = Field(
        None,
        alias="hostPID",
        description="hostPID determines if the policy allows the use of HostPID in the pod spec.",
    )
    host_ports: Optional[List[HostPortRange]] = Field(
        None,
        alias="hostPorts",
        description="hostPorts determines which host port ranges are allowed to be exposed.",
    )
    privileged: Optional[bool] = Field(
        None,
        description="privileged determines if a pod can request to be run as privileged.",
    )
    read_only_root_filesystem: Optional[bool] = Field(
        None,
        alias="readOnlyRootFilesystem",
        description="readOnlyRootFilesystem when set to true will force containers to run with a read only root file system.  If the container specifically requests to run with a non-read only root file system the PSP should deny the pod. If set to false the container may run with a read only root file system if it wishes but it will not be forced to.",
    )
    required_drop_capabilities: Optional[List[str]] = Field(
        None,
        alias="requiredDropCapabilities",
        description="requiredDropCapabilities are the capabilities that will be dropped from the container.  These are required to be dropped and cannot be added.",
    )
    run_as_group: Optional[RunAsGroupStrategyOptions] = Field(
        None,
        alias="runAsGroup",
        description="RunAsGroup is the strategy that will dictate the allowable RunAsGroup values that may be set. If this field is omitted, the pod's RunAsGroup can take any value. This field requires the RunAsGroup feature gate to be enabled.",
    )
    run_as_user: RunAsUserStrategyOptions = Field(
        ...,
        alias="runAsUser",
        description="runAsUser is the strategy that will dictate the allowable RunAsUser values that may be set.",
    )
    runtime_class: Optional[RuntimeClassStrategyOptions] = Field(
        None,
        alias="runtimeClass",
        description="runtimeClass is the strategy that will dictate the allowable RuntimeClasses for a pod. If this field is omitted, the pod's runtimeClassName field is unrestricted. Enforcement of this field depends on the RuntimeClass feature gate being enabled.",
    )
    se_linux: SELinuxStrategyOptions = Field(
        ...,
        alias="seLinux",
        description="seLinux is the strategy that will dictate the allowable labels that may be set.",
    )
    supplemental_groups: SupplementalGroupsStrategyOptions = Field(
        ...,
        alias="supplementalGroups",
        description="supplementalGroups is the strategy that will dictate what supplemental groups are used by the SecurityContext.",
    )
    volumes: Optional[List[str]] = Field(
        None,
        description="volumes is a white list of allowed volume plugins. Empty indicates that no volumes may be used. To allow all volumes you may use '*'.",
    )


class ReplicaSetCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="The last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of replica set condition.")


class ReplicaSetStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    available_replicas: Optional[int] = Field(
        None,
        alias="availableReplicas",
        description="The number of available replicas (ready for at least minReadySeconds) for this replica set.",
    )
    conditions: Optional[List[ReplicaSetCondition]] = Field(
        None,
        description="Represents the latest available observations of a replica set's current state.",
    )
    fully_labeled_replicas: Optional[int] = Field(
        None,
        alias="fullyLabeledReplicas",
        description="The number of pods that have labels matching the labels of the pod template of the replicaset.",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
    )
    ready_replicas: Optional[int] = Field(
        None,
        alias="readyReplicas",
        description="The number of ready replicas for this replica set.",
    )
    replicas: int = Field(
        ...,
        description="Replicas is the most recently oberved number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
    )


class RollingUpdateDaemonSet(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_unavailable: Optional[Union[int, str]] = Field(
        None,
        alias="maxUnavailable",
        description="The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0. Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
    )


class RollingUpdateDeployment(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_surge: Optional[Union[int, str]] = Field(
        None,
        alias="maxSurge",
        description="The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. By default, a value of 1 is used. Example: when this is set to 30%, the new RC can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new RC can be scaled up further, ensuring that total number of pods running at any time during the update is at most 130% of desired pods.",
    )
    max_unavailable: Optional[Union[int, str]] = Field(
        None,
        alias="maxUnavailable",
        description="The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. By default, a fixed value of 1 is used. Example: when this is set to 30%, the old RC can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old RC can be scaled down further, followed by scaling up the new RC, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
    )


class DaemonSetUpdateStrategy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rolling_update: Optional[RollingUpdateDaemonSet] = Field(
        None,
        alias="rollingUpdate",
        description='Rolling update config params. Present only if type = "RollingUpdate".',
    )
    type: Optional[str] = Field(
        None,
        description='Type of daemon set update. Can be "RollingUpdate" or "OnDelete". Default is OnDelete.',
    )


class DeploymentStrategy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rolling_update: Optional[RollingUpdateDeployment] = Field(
        None,
        alias="rollingUpdate",
        description="Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
    )
    type: Optional[str] = Field(
        None,
        description='Type of deployment. Can be "Recreate" or "RollingUpdate". Default is RollingUpdate.',
    )


class HTTPIngressPath(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    backend: IngressBackend = Field(
        ...,
        description="Backend defines the referenced service endpoint to which the traffic will be forwarded to.",
    )
    path: Optional[str] = Field(
        None,
        description="Path is an extended POSIX regex as defined by IEEE Std 1003.1, (i.e this follows the egrep/unix syntax, not the perl syntax) matched against the path of an incoming request. Currently it can contain characters disallowed from the conventional \"path\" part of a URL as defined by RFC 3986. Paths must begin with a '/'. If unspecified, the path defaults to a catch all sending traffic to the backend.",
    )


class HTTPIngressRuleValue(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    paths: List[HTTPIngressPath] = Field(
        ..., description="A collection of paths that map requests to backends."
    )


class IngressRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    host: Optional[str] = Field(
        None,
        description='Host is the fully qualified domain name of a network host, as defined by RFC 3986. Note the following deviations from the "host" part of the URI as defined in the RFC: 1. IPs are not allowed. Currently an IngressRuleValue can only apply to the\n\t  IP in the Spec of the parent Ingress.\n2. The `:` delimiter is not respected because ports are not allowed.\n\t  Currently the port of an Ingress is implicitly :80 for http and\n\t  :443 for https.\nBoth these may change in the future. Incoming requests are matched against the host before the IngressRuleValue. If the host is unspecified, the Ingress routes all traffic based on the specified IngressRuleValue.',
    )
    http: Optional[HTTPIngressRuleValue] = None


class IngressSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    backend: Optional[IngressBackend] = Field(
        None,
        description="A default backend capable of servicing requests that don't match any rule. At least one of 'backend' or 'rules' must be specified. This field is optional to allow the loadbalancer controller or defaulting logic to specify a global default.",
    )
    rules: Optional[List[IngressRule]] = Field(
        None,
        description="A list of host rules used to configure the Ingress. If unspecified, or no rule matches, all traffic is sent to the default backend.",
    )
    tls: Optional[List[IngressTLS]] = Field(
        None,
        description="TLS configuration. Currently the Ingress only supports a single TLS port, 443. If multiple members of this list specify different hosts, they will be multiplexed on the same port according to the hostname specified through the SNI TLS extension, if the ingress controller fulfilling the ingress supports SNI.",
    )


class NetworkPolicyPeer(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ip_block: Optional[IPBlock] = Field(
        None,
        alias="ipBlock",
        description="IPBlock defines policy on a particular IPBlock. If this field is set then neither of the other fields can be.",
    )
    namespace_selector: Optional[v1.LabelSelector] = Field(
        None,
        alias="namespaceSelector",
        description="Selects Namespaces using cluster-scoped labels. This field follows standard label selector semantics; if present but empty, it selects all namespaces.\n\nIf PodSelector is also set, then the NetworkPolicyPeer as a whole selects the Pods matching PodSelector in the Namespaces selected by NamespaceSelector. Otherwise it selects all Pods in the Namespaces selected by NamespaceSelector.",
    )
    pod_selector: Optional[v1.LabelSelector] = Field(
        None,
        alias="podSelector",
        description="This is a label selector which selects Pods. This field follows standard label selector semantics; if present but empty, it selects all pods.\n\nIf NamespaceSelector is also set, then the NetworkPolicyPeer as a whole selects the Pods matching PodSelector in the Namespaces selected by NamespaceSelector. Otherwise it selects the Pods matching PodSelector in the policy's own Namespace.",
    )


class PodSecurityPolicy(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind115] = Field(
        "PodSecurityPolicy",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[PodSecurityPolicySpec] = Field(
        None, description="spec defines the policy enforced."
    )


class PodSecurityPolicyList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[PodSecurityPolicy] = Field(
        ..., description="items is a list of schema objects."
    )
    kind: Optional[Kind116] = Field(
        "PodSecurityPolicyList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class Scale(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind119] = Field(
        "Scale",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.",
    )
    spec: Optional[ScaleSpec] = Field(
        None,
        description="defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.",
    )


class Ingress(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind111] = Field(
        "Ingress",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[IngressSpec] = Field(
        None,
        description="Spec is the desired state of the Ingress. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class IngressList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Ingress] = Field(..., description="Items is the list of Ingress.")
    kind: Optional[Kind112] = Field(
        "IngressList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class NetworkPolicyEgressRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ports: Optional[List[NetworkPolicyPort]] = Field(
        None,
        description="List of destination ports for outgoing traffic. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
    )
    to: Optional[List[NetworkPolicyPeer]] = Field(
        None,
        description="List of destinations for outgoing traffic of pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all destinations (traffic not restricted by destination). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the to list.",
    )


class NetworkPolicyIngressRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    from_: Optional[List[NetworkPolicyPeer]] = Field(
        None,
        alias="from",
        description="List of sources which should be able to access the pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all sources (traffic not restricted by source). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the from list.",
    )
    ports: Optional[List[NetworkPolicyPort]] = Field(
        None,
        description="List of ports which should be made accessible on the pods selected for this rule. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
    )


class NetworkPolicySpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    egress: Optional[List[NetworkPolicyEgressRule]] = Field(
        None,
        description="List of egress rules to be applied to the selected pods. Outgoing traffic is allowed if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic matches at least one egress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy limits all outgoing traffic (and serves solely to ensure that the pods it selects are isolated by default). This field is beta-level in 1.8",
    )
    ingress: Optional[List[NetworkPolicyIngressRule]] = Field(
        None,
        description="List of ingress rules to be applied to the selected pods. Traffic is allowed to a pod if there are no NetworkPolicies selecting the pod OR if the traffic source is the pod's local node, OR if the traffic matches at least one ingress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy does not allow any traffic (and serves solely to ensure that the pods it selects are isolated by default).",
    )
    pod_selector: v1.LabelSelector = Field(
        ...,
        alias="podSelector",
        description="Selects the pods to which this NetworkPolicy object applies.  The array of ingress rules is applied to any pods selected by this field. Multiple network policies can select the same set of pods.  In this case, the ingress rules for each are combined additively. This field is NOT optional and follows standard label selector semantics. An empty podSelector matches all pods in this namespace.",
    )
    policy_types: Optional[List[str]] = Field(
        None,
        alias="policyTypes",
        description='List of rule types that the NetworkPolicy relates to. Valid options are "Ingress", "Egress", or "Ingress,Egress". If this field is not specified, it will default based on the existence of Ingress or Egress rules; policies that contain an Egress section are assumed to affect Egress, and all policies (whether or not they contain an Ingress section) are assumed to affect Ingress. If you want to write an egress-only policy, you must explicitly specify policyTypes [ "Egress" ]. Likewise, if you want to write a policy that specifies that no egress is allowed, you must specify a policyTypes value that include "Egress" (since such a policy would not include an Egress section and would otherwise default to just [ "Ingress" ]). This field is beta-level in 1.8',
    )


class NetworkPolicy(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind113] = Field(
        "NetworkPolicy",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[NetworkPolicySpec] = Field(
        None,
        description="Specification of the desired behavior for this NetworkPolicy.",
    )


class NetworkPolicyList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[NetworkPolicy] = Field(
        ..., description="Items is a list of schema objects."
    )
    kind: Optional[Kind114] = Field(
        "NetworkPolicyList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class DaemonSetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    min_ready_seconds: Optional[int] = Field(
        None,
        alias="minReadySeconds",
        description="The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
    )
    revision_history_limit: Optional[int] = Field(
        None,
        alias="revisionHistoryLimit",
        description="The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="A label query over pods that are managed by the daemon set. Must match in order to be controlled. If empty, defaulted to labels on Pod template. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )
    template: v1_1.PodTemplateSpec = Field(
        ...,
        description="An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template's node selector (or on every node if no node selector is specified). More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
    )
    template_generation: Optional[int] = Field(
        None,
        alias="templateGeneration",
        description="DEPRECATED. A sequence number representing a specific generation of the template. Populated by the system. It can be set only during the creation.",
    )
    update_strategy: Optional[DaemonSetUpdateStrategy] = Field(
        None,
        alias="updateStrategy",
        description="An update strategy to replace existing DaemonSet pods with new pods.",
    )


class DeploymentSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    min_ready_seconds: Optional[int] = Field(
        None,
        alias="minReadySeconds",
        description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
    )
    paused: Optional[bool] = Field(
        None,
        description="Indicates that the deployment is paused and will not be processed by the deployment controller.",
    )
    progress_deadline_seconds: Optional[int] = Field(
        None,
        alias="progressDeadlineSeconds",
        description='The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. This is set to the max value of int32 (i.e. 2147483647) by default, which means "no deadline".',
    )
    replicas: Optional[int] = Field(
        None,
        description="Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
    )
    revision_history_limit: Optional[int] = Field(
        None,
        alias="revisionHistoryLimit",
        description='The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. This is set to the max value of int32 (i.e. 2147483647) by default, which means "retaining all old RelicaSets".',
    )
    rollback_to: Optional[RollbackConfig] = Field(
        None,
        alias="rollbackTo",
        description="DEPRECATED. The config this deployment is rolling back to. Will be cleared after rollback is done.",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment.",
    )
    strategy: Optional[DeploymentStrategy] = Field(
        None,
        description="The deployment strategy to use to replace existing pods with new ones.",
    )
    template: v1_1.PodTemplateSpec = Field(
        ..., description="Template describes the pods that will be created."
    )


class ReplicaSetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    min_ready_seconds: Optional[int] = Field(
        None,
        alias="minReadySeconds",
        description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
    )
    replicas: Optional[int] = Field(
        None,
        description="Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="Selector is a label query over pods that should match the replica count. If the selector is empty, it is defaulted to the labels present on the pod template. Label keys and values that must match in order to be controlled by this replica set. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )
    template: Optional[v1_1.PodTemplateSpec] = Field(
        None,
        description="Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
    )


class DaemonSet(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind106] = Field(
        "DaemonSet",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[DaemonSetSpec] = Field(
        None,
        description="The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class DaemonSetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[DaemonSet] = Field(..., description="A list of daemon sets.")
    kind: Optional[Kind107] = Field(
        "DaemonSetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class Deployment(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind108] = Field(
        "Deployment",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object metadata."
    )
    spec: Optional[DeploymentSpec] = Field(
        None, description="Specification of the desired behavior of the Deployment."
    )


class DeploymentList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Deployment] = Field(
        ..., description="Items is the list of Deployments."
    )
    kind: Optional[Kind109] = Field(
        "DeploymentList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(None, description="Standard list metadata.")


class ReplicaSet(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind117] = Field(
        "ReplicaSet",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[ReplicaSetSpec] = Field(
        None,
        description="Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class ReplicaSetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "extensions/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[ReplicaSet] = Field(
        ...,
        description="List of ReplicaSets. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller",
    )
    kind: Optional[Kind118] = Field(
        "ReplicaSetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind37, Kind38
from ...apimachinery.pkg.apis.meta import v1


class Policy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    level: str = Field(
        ...,
        description="The Level that all requests are recorded at. available options: None, Metadata, Request, RequestResponse required",
    )
    stages: Optional[List[str]] = Field(
        None, description="Stages is a list of stages for which events are created."
    )


class ServiceReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: str = Field(..., description="`name` is the name of the service. Required")
    namespace: str = Field(
        ..., description="`namespace` is the namespace of the service. Required"
    )
    path: Optional[str] = Field(
        None,
        description="`path` is an optional URL path which will be sent in any request to this service.",
    )
    port: Optional[int] = Field(
        None,
        description="If specified, the port on the service that hosting webhook. Default to 443 for backward compatibility. `port` should be a valid port number (1-65535, inclusive).",
    )


class WebhookClientConfig(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ca_bundle: Optional[str] = Field(
        None,
        alias="caBundle",
        description="`caBundle` is a PEM encoded CA bundle which will be used to validate the webhook's server certificate. If unspecified, system trust roots on the apiserver are used.",
    )
    service: Optional[ServiceReference] = Field(
        None,
        description="`service` is a reference to the service for this webhook. Either `service` or `url` must be specified.\n\nIf the webhook is running within the cluster, then you should use `service`.",
    )
    url: Optional[str] = Field(
        None,
        description='`url` gives the location of the webhook, in standard URL form (`scheme://host:port/path`). Exactly one of `url` or `service` must be specified.\n\nThe `host` should not refer to a service running in the cluster; use the `service` field instead. The host might be resolved via external DNS in some apiservers (e.g., `kube-apiserver` cannot resolve in-cluster DNS as that would be a layering violation). `host` may also be an IP address.\n\nPlease note that using `localhost` or `127.0.0.1` as a `host` is risky unless you take great care to run this webhook on all hosts which run an apiserver which might need to make calls to this webhook. Such installs are likely to be non-portable, i.e., not easy to turn up in a new cluster.\n\nThe scheme must be "https"; the URL must begin with "https://".\n\nA path is optional, and if present may be any string permissible in a URL. You may use the path to pass an arbitrary string to the webhook, for example, a cluster identifier.\n\nAttempting to use a user or basic auth e.g. "user:password@" is not allowed. Fragments ("#...") and query parameters ("?...") are not allowed, either.',
    )


class WebhookThrottleConfig(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    burst: Optional[int] = Field(
        None,
        description="ThrottleBurst is the maximum number of events sent at the same moment default 15 QPS",
    )
    qps: Optional[int] = Field(
        None,
        description="ThrottleQPS maximum number of batches per second default 10 QPS",
    )


class Webhook(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    client_config: WebhookClientConfig = Field(
        ...,
        alias="clientConfig",
        description="ClientConfig holds the connection parameters for the webhook required",
    )
    throttle: Optional[WebhookThrottleConfig] = Field(
        None, description="Throttle holds the options for throttling the webhook"
    )


class AuditSinkSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    policy: Policy = Field(
        ...,
        description="Policy defines the policy for selecting which events should be sent to the webhook required",
    )
    webhook: Webhook = Field(..., description="Webhook to send events required")


class AuditSink(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "auditregistration.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind37] = Field(
        "AuditSink",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: Optional[AuditSinkSpec] = Field(
        None, description="Spec defines the audit configuration spec"
    )


class AuditSinkList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "auditregistration.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[AuditSink] = Field(..., description="List of audit configurations.")
    kind: Optional[Kind38] = Field(
        "AuditSinkList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = None



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind128, Kind129, Kind130, Kind131, Kind132
from ...apimachinery.pkg.apis.meta import v1
from ...apimachinery.pkg.util import intstr
from ..core import v1


class AllowedCSIDriver(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: str = Field(..., description="Name is the registered name of the CSI driver")


class AllowedFlexVolume(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    driver: str = Field(..., description="driver is the name of the Flexvolume driver.")


class AllowedHostPath(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    path_prefix: Optional[str] = Field(
        None,
        alias="pathPrefix",
        description="pathPrefix is the path prefix that the host volume must match. It does not support `*`. Trailing slashes are trimmed when validating the path prefix with a host path.\n\nExamples: `/foo` would allow `/foo`, `/foo/` and `/foo/bar` `/foo` would not allow `/food` or `/etc/foo`",
    )
    read_only: Optional[bool] = Field(
        None,
        alias="readOnly",
        description="when set to true, will allow host volumes matching the pathPrefix only if all volume mounts are readOnly.",
    )


class HostPortRange(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max: int = Field(..., description="max is the end of the range, inclusive.")
    min: int = Field(..., description="min is the start of the range, inclusive.")


class IDRange(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max: int = Field(..., description="max is the end of the range, inclusive.")
    min: int = Field(..., description="min is the start of the range, inclusive.")


class PodDisruptionBudgetStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    current_healthy: int = Field(
        ..., alias="currentHealthy", description="current number of healthy pods"
    )
    desired_healthy: int = Field(
        ...,
        alias="desiredHealthy",
        description="minimum desired number of healthy pods",
    )
    disrupted_pods: Optional[Dict[str, Any]] = Field(
        None,
        alias="disruptedPods",
        description="DisruptedPods contains information about pods whose eviction was processed by the API server eviction subresource handler but has not yet been observed by the PodDisruptionBudget controller. A pod will be in this map from the time when the API server processed the eviction request to the time when the pod is seen by PDB controller as having been marked for deletion (or after a timeout). The key in the map is the name of the pod and the value is the time when the API server processed the eviction request. If the deletion didn't occur and a pod is still there it will be removed from the list automatically by PodDisruptionBudget controller after some time. If everything goes smooth this map should be empty for the most of the time. Large number of entries in the map may indicate problems with pod deletions.",
    )
    disruptions_allowed: int = Field(
        ...,
        alias="disruptionsAllowed",
        description="Number of pod disruptions that are currently allowed.",
    )
    expected_pods: int = Field(
        ...,
        alias="expectedPods",
        description="total number of pods counted by this disruption budget",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="Most recent generation observed when updating this PDB status. PodDisruptionsAllowed and other status informatio is valid only if observedGeneration equals to PDB's object generation.",
    )


class RunAsGroupStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of gids that may be used. If you would like to force a single gid then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: str = Field(
        ...,
        description="rule is the strategy that will dictate the allowable RunAsGroup values that may be set.",
    )


class RunAsUserStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of uids that may be used. If you would like to force a single uid then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: str = Field(
        ...,
        description="rule is the strategy that will dictate the allowable RunAsUser values that may be set.",
    )


class RuntimeClassStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    allowed_runtime_class_names: List[str] = Field(
        ...,
        alias="allowedRuntimeClassNames",
        description='allowedRuntimeClassNames is a whitelist of RuntimeClass names that may be specified on a pod. A value of "*" means that any RuntimeClass name is allowed, and must be the only item in the list. An empty list requires the RuntimeClassName field to be unset.',
    )
    default_runtime_class_name: Optional[str] = Field(
        None,
        alias="defaultRuntimeClassName",
        description="defaultRuntimeClassName is the default RuntimeClassName to set on the pod. The default MUST be allowed by the allowedRuntimeClassNames list. A value of nil does not mutate the Pod.",
    )


class SELinuxStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rule: str = Field(
        ...,
        description="rule is the strategy that will dictate the allowable labels that may be set.",
    )
    se_linux_options: Optional[v1_1.SELinuxOptions] = Field(
        None,
        alias="seLinuxOptions",
        description="seLinuxOptions required to run as; required for MustRunAs More info: https://kubernetes.io/docs/tasks/configure-pod-container/security-context/",
    )


class SupplementalGroupsStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of supplemental groups.  If you would like to force a single supplemental group then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: Optional[str] = Field(
        None,
        description="rule is the strategy that will dictate what supplemental groups is used in the SecurityContext.",
    )


class FSGroupStrategyOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ranges: Optional[List[IDRange]] = Field(
        None,
        description="ranges are the allowed ranges of fs groups.  If you would like to force a single fs group then supply a single range with the same start and end. Required for MustRunAs.",
    )
    rule: Optional[str] = Field(
        None,
        description="rule is the strategy that will dictate what FSGroup is used in the SecurityContext.",
    )


class PodSecurityPolicySpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    allow_privilege_escalation: Optional[bool] = Field(
        None,
        alias="allowPrivilegeEscalation",
        description="allowPrivilegeEscalation determines if a pod can request to allow privilege escalation. If unspecified, defaults to true.",
    )
    allowed_csi_drivers: Optional[List[AllowedCSIDriver]] = Field(
        None,
        alias="allowedCSIDrivers",
        description="AllowedCSIDrivers is a whitelist of inline CSI drivers that must be explicitly set to be embedded within a pod spec. An empty value indicates that any CSI driver can be used for inline ephemeral volumes. This is an alpha field, and is only honored if the API server enables the CSIInlineVolume feature gate.",
    )
    allowed_capabilities: Optional[List[str]] = Field(
        None,
        alias="allowedCapabilities",
        description="allowedCapabilities is a list of capabilities that can be requested to add to the container. Capabilities in this field may be added at the pod author's discretion. You must not list a capability in both allowedCapabilities and requiredDropCapabilities.",
    )
    allowed_flex_volumes: Optional[List[AllowedFlexVolume]] = Field(
        None,
        alias="allowedFlexVolumes",
        description='allowedFlexVolumes is a whitelist of allowed Flexvolumes.  Empty or nil indicates that all Flexvolumes may be used.  This parameter is effective only when the usage of the Flexvolumes is allowed in the "volumes" field.',
    )
    allowed_host_paths: Optional[List[AllowedHostPath]] = Field(
        None,
        alias="allowedHostPaths",
        description="allowedHostPaths is a white list of allowed host paths. Empty indicates that all host paths may be used.",
    )
    allowed_proc_mount_types: Optional[List[str]] = Field(
        None,
        alias="allowedProcMountTypes",
        description="AllowedProcMountTypes is a whitelist of allowed ProcMountTypes. Empty or nil indicates that only the DefaultProcMountType may be used. This requires the ProcMountType feature flag to be enabled.",
    )
    allowed_unsafe_sysctls: Optional[List[str]] = Field(
        None,
        alias="allowedUnsafeSysctls",
        description='allowedUnsafeSysctls is a list of explicitly allowed unsafe sysctls, defaults to none. Each entry is either a plain sysctl name or ends in "*" in which case it is considered as a prefix of allowed sysctls. Single * means all unsafe sysctls are allowed. Kubelet has to whitelist all allowed unsafe sysctls explicitly to avoid rejection.\n\nExamples: e.g. "foo/*" allows "foo/bar", "foo/baz", etc. e.g. "foo.*" allows "foo.bar", "foo.baz", etc.',
    )
    default_add_capabilities: Optional[List[str]] = Field(
        None,
        alias="defaultAddCapabilities",
        description="defaultAddCapabilities is the default set of capabilities that will be added to the container unless the pod spec specifically drops the capability.  You may not list a capability in both defaultAddCapabilities and requiredDropCapabilities. Capabilities added here are implicitly allowed, and need not be included in the allowedCapabilities list.",
    )
    default_allow_privilege_escalation: Optional[bool] = Field(
        None,
        alias="defaultAllowPrivilegeEscalation",
        description="defaultAllowPrivilegeEscalation controls the default setting for whether a process can gain more privileges than its parent process.",
    )
    forbidden_sysctls: Optional[List[str]] = Field(
        None,
        alias="forbiddenSysctls",
        description='forbiddenSysctls is a list of explicitly forbidden sysctls, defaults to none. Each entry is either a plain sysctl name or ends in "*" in which case it is considered as a prefix of forbidden sysctls. Single * means all sysctls are forbidden.\n\nExamples: e.g. "foo/*" forbids "foo/bar", "foo/baz", etc. e.g. "foo.*" forbids "foo.bar", "foo.baz", etc.',
    )
    fs_group: FSGroupStrategyOptions = Field(
        ...,
        alias="fsGroup",
        description="fsGroup is the strategy that will dictate what fs group is used by the SecurityContext.",
    )
    host_ipc: Optional[bool] = Field(
        None,
        alias="hostIPC",
        description="hostIPC determines if the policy allows the use of HostIPC in the pod spec.",
    )
    host_network: Optional[bool] = Field(
        None,
        alias="hostNetwork",
        description="hostNetwork determines if the policy allows the use of HostNetwork in the pod spec.",
    )
    host_pid: Optional[bool] = Field(
        None,
        alias="hostPID",
        description="hostPID determines if the policy allows the use of HostPID in the pod spec.",
    )
    host_ports: Optional[List[HostPortRange]] = Field(
        None,
        alias="hostPorts",
        description="hostPorts determines which host port ranges are allowed to be exposed.",
    )
    privileged: Optional[bool] = Field(
        None,
        description="privileged determines if a pod can request to be run as privileged.",
    )
    read_only_root_filesystem: Optional[bool] = Field(
        None,
        alias="readOnlyRootFilesystem",
        description="readOnlyRootFilesystem when set to true will force containers to run with a read only root file system.  If the container specifically requests to run with a non-read only root file system the PSP should deny the pod. If set to false the container may run with a read only root file system if it wishes but it will not be forced to.",
    )
    required_drop_capabilities: Optional[List[str]] = Field(
        None,
        alias="requiredDropCapabilities",
        description="requiredDropCapabilities are the capabilities that will be dropped from the container.  These are required to be dropped and cannot be added.",
    )
    run_as_group: Optional[RunAsGroupStrategyOptions] = Field(
        None,
        alias="runAsGroup",
        description="RunAsGroup is the strategy that will dictate the allowable RunAsGroup values that may be set. If this field is omitted, the pod's RunAsGroup can take any value. This field requires the RunAsGroup feature gate to be enabled.",
    )
    run_as_user: RunAsUserStrategyOptions = Field(
        ...,
        alias="runAsUser",
        description="runAsUser is the strategy that will dictate the allowable RunAsUser values that may be set.",
    )
    runtime_class: Optional[RuntimeClassStrategyOptions] = Field(
        None,
        alias="runtimeClass",
        description="runtimeClass is the strategy that will dictate the allowable RuntimeClasses for a pod. If this field is omitted, the pod's runtimeClassName field is unrestricted. Enforcement of this field depends on the RuntimeClass feature gate being enabled.",
    )
    se_linux: SELinuxStrategyOptions = Field(
        ...,
        alias="seLinux",
        description="seLinux is the strategy that will dictate the allowable labels that may be set.",
    )
    supplemental_groups: SupplementalGroupsStrategyOptions = Field(
        ...,
        alias="supplementalGroups",
        description="supplementalGroups is the strategy that will dictate what supplemental groups are used by the SecurityContext.",
    )
    volumes: Optional[List[str]] = Field(
        None,
        description="volumes is a white list of allowed volume plugins. Empty indicates that no volumes may be used. To allow all volumes you may use '*'.",
    )


class Eviction(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "policy/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    delete_options: Optional[v1.DeleteOptions] = Field(
        None, alias="deleteOptions", description="DeleteOptions may be provided"
    )
    kind: Optional[Kind128] = Field(
        "Eviction",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="ObjectMeta describes the pod that is being evicted."
    )


class PodDisruptionBudgetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_unavailable: Optional[Union[int, str]] = Field(
        None,
        alias="maxUnavailable",
        description='An eviction is allowed if at most "maxUnavailable" pods selected by "selector" are unavailable after the eviction, i.e. even in absence of the evicted pod. For example, one can prevent all voluntary evictions by specifying 0. This is a mutually exclusive setting with "minAvailable".',
    )
    min_available: Optional[Union[int, str]] = Field(
        None,
        alias="minAvailable",
        description='An eviction is allowed if at least "minAvailable" pods selected by "selector" will still be available after the eviction, i.e. even in the absence of the evicted pod.  So for example you can prevent all voluntary evictions by specifying "100%".',
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="Label query over pods whose evictions are managed by the disruption budget.",
    )


class PodSecurityPolicy(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "policy/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind131] = Field(
        "PodSecurityPolicy",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[PodSecurityPolicySpec] = Field(
        None, description="spec defines the policy enforced."
    )


class PodSecurityPolicyList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "policy/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[PodSecurityPolicy] = Field(
        ..., description="items is a list of schema objects."
    )
    kind: Optional[Kind132] = Field(
        "PodSecurityPolicyList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class PodDisruptionBudget(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "policy/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind129] = Field(
        "PodDisruptionBudget",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: Optional[PodDisruptionBudgetSpec] = Field(
        None,
        description="Specification of the desired behavior of the PodDisruptionBudget.",
    )


class PodDisruptionBudgetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "policy/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[PodDisruptionBudget]
    kind: Optional[Kind130] = Field(
        "PodDisruptionBudgetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = None



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind165, Kind166, Kind167, Kind168
from ...apimachinery.pkg.apis.meta import v1 as v1_1
from ..core import v1


class VolumeError(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    message: Optional[str] = Field(
        None,
        description="String detailing the error encountered during Attach or Detach operation. This string may be logged, so it should not contain sensitive information.",
    )
    time: Optional[v1.Time] = Field(None, description="Time the error was encountered.")


class StorageClass(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    allow_volume_expansion: Optional[bool] = Field(
        None,
        alias="allowVolumeExpansion",
        description="AllowVolumeExpansion shows whether the storage class allow volume expand",
    )
    allowed_topologies: Optional[List[v1.TopologySelectorTerm]] = Field(
        None,
        alias="allowedTopologies",
        description="Restrict the node topologies where volumes can be dynamically provisioned. Each volume plugin defines its own supported topology specifications. An empty TopologySelectorTerm list means there is no topology restriction. This field is only honored by servers that enable the VolumeScheduling feature.",
    )
    api_version: Optional[str] = Field(
        "storage.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind165] = Field(
        "StorageClass",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1_1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    mount_options: Optional[List[str]] = Field(
        None,
        alias="mountOptions",
        description='Dynamically provisioned PersistentVolumes of this storage class are created with these mountOptions, e.g. ["ro", "soft"]. Not validated - mount of the PVs will simply fail if one is invalid.',
    )
    parameters: Optional[Dict[str, Any]] = Field(
        None,
        description="Parameters holds the parameters for the provisioner that should create volumes of this storage class.",
    )
    provisioner: str = Field(
        ..., description="Provisioner indicates the type of the provisioner."
    )
    reclaim_policy: Optional[str] = Field(
        None,
        alias="reclaimPolicy",
        description="Dynamically provisioned PersistentVolumes of this storage class are created with this reclaimPolicy. Defaults to Delete.",
    )
    volume_binding_mode: Optional[str] = Field(
        None,
        alias="volumeBindingMode",
        description="VolumeBindingMode indicates how PersistentVolumeClaims should be provisioned and bound.  When unset, VolumeBindingImmediate is used. This field is only honored by servers that enable the VolumeScheduling feature.",
    )


class StorageClassList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "storage.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[StorageClass] = Field(
        ..., description="Items is the list of StorageClasses"
    )
    kind: Optional[Kind166] = Field(
        "StorageClassList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class VolumeAttachmentSource(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    inline_volume_spec: Optional[v1.PersistentVolumeSpec] = Field(
        None,
        alias="inlineVolumeSpec",
        description="inlineVolumeSpec contains all the information necessary to attach a persistent volume defined by a pod's inline VolumeSource. This field is populated only for the CSIMigration feature. It contains translated fields from a pod's inline VolumeSource to a PersistentVolumeSpec. This field is alpha-level and is only honored by servers that enabled the CSIMigration feature.",
    )
    persistent_volume_name: Optional[str] = Field(
        None,
        alias="persistentVolumeName",
        description="Name of the persistent volume to attach.",
    )


class VolumeAttachmentSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    attacher: str = Field(
        ...,
        description="Attacher indicates the name of the volume driver that MUST handle this request. This is the name returned by GetPluginName().",
    )
    node_name: str = Field(
        ...,
        alias="nodeName",
        description="The node that the volume should be attached to.",
    )
    source: VolumeAttachmentSource = Field(
        ..., description="Source represents the volume that should be attached."
    )


class VolumeAttachmentStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    attach_error: Optional[VolumeError] = Field(
        None,
        alias="attachError",
        description="The last error encountered during attach operation, if any. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
    )
    attached: bool = Field(
        ...,
        description="Indicates the volume is successfully attached. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
    )
    attachment_metadata: Optional[Dict[str, Any]] = Field(
        None,
        alias="attachmentMetadata",
        description="Upon successful attach, this field is populated with any information returned by the attach operation that must be passed into subsequent WaitForAttach or Mount calls. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
    )
    detach_error: Optional[VolumeError] = Field(
        None,
        alias="detachError",
        description="The last error encountered during detach operation, if any. This field must only be set by the entity completing the detach operation, i.e. the external-attacher.",
    )


class VolumeAttachment(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "storage.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind167] = Field(
        "VolumeAttachment",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: VolumeAttachmentSpec = Field(
        ...,
        description="Specification of the desired attach/detach volume behavior. Populated by the Kubernetes system.",
    )


class VolumeAttachmentList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "storage.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[VolumeAttachment] = Field(
        ..., description="Items is the list of VolumeAttachments"
    )
    kind: Optional[Kind168] = Field(
        "VolumeAttachmentList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind159, Kind160
from ...apimachinery.pkg.apis.meta import v1


class PriorityClass(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "scheduling.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    description: Optional[str] = Field(
        None,
        description="description is an arbitrary string that usually provides guidelines on when this priority class should be used.",
    )
    global_default: Optional[bool] = Field(
        None,
        alias="globalDefault",
        description="globalDefault specifies whether this PriorityClass should be considered as the default priority for pods that do not have any priority class. Only one PriorityClass can be marked as `globalDefault`. However, if more than one PriorityClasses exists with their `globalDefault` field set to true, the smallest value of such global default PriorityClasses will be used as the default priority.",
    )
    kind: Optional[Kind159] = Field(
        "PriorityClass",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    preemption_policy: Optional[str] = Field(
        None,
        alias="preemptionPolicy",
        description="PreemptionPolicy is the Policy for preempting pods with lower priority. One of Never, PreemptLowerPriority. Defaults to PreemptLowerPriority if unset. This field is alpha-level and is only honored by servers that enable the NonPreemptingPriority feature.",
    )
    value: int = Field(
        ...,
        description="The value of this priority class. This is the actual priority that pods receive when they have the name of this class in their pod spec.",
    )


class PriorityClassList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "scheduling.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[PriorityClass] = Field(
        ..., description="items is the list of PriorityClasses"
    )
    kind: Optional[Kind160] = Field(
        "PriorityClassList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ....... import Kind181, Kind182
from .....apimachinery.pkg.apis.meta import v1


class CustomResourceColumnDefinition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    json_path: str = Field(
        ...,
        alias="JSONPath",
        description="JSONPath is a simple JSON path (i.e. with array notation) which is evaluated against each custom resource to produce the value for this column.",
    )
    description: Optional[str] = Field(
        None, description="description is a human readable description of this column."
    )
    format: Optional[str] = Field(
        None,
        description="format is an optional OpenAPI type definition for this column. The 'name' format is applied to the primary identifier column to assist in clients identifying column is the resource name. See https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types for details.",
    )
    name: str = Field(..., description="name is a human readable name for the column.")
    priority: Optional[int] = Field(
        None,
        description="priority is an integer defining the relative importance of this column compared to others. Lower numbers are considered higher priority. Columns that may be omitted in limited space scenarios should be given a priority greater than 0.",
    )
    type: str = Field(
        ...,
        description="type is an OpenAPI type definition for this column. See https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types for details.",
    )


class CustomResourceDefinitionNames(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    categories: Optional[List[str]] = Field(
        None,
        description="categories is a list of grouped resources this custom resource belongs to (e.g. 'all'). This is published in API discovery documents, and used by clients to support invocations like `kubectl get all`.",
    )
    kind: str = Field(
        ...,
        description="kind is the serialized kind of the resource. It is normally CamelCase and singular. Custom resource instances will use this value as the `kind` attribute in API calls.",
    )
    list_kind: Optional[str] = Field(
        None,
        alias="listKind",
        description='listKind is the serialized kind of the list for this resource. Defaults to "`kind`List".',
    )
    plural: str = Field(
        ...,
        description="plural is the plural name of the resource to serve. The custom resources are served under `/apis/<group>/<version>/.../<plural>`. Must match the name of the CustomResourceDefinition (in the form `<names.plural>.<group>`). Must be all lowercase.",
    )
    short_names: Optional[List[str]] = Field(
        None,
        alias="shortNames",
        description="shortNames are short names for the resource, exposed in API discovery documents, and used by clients to support invocations like `kubectl get <shortname>`. It must be all lowercase.",
    )
    singular: Optional[str] = Field(
        None,
        description="singular is the singular name of the resource. It must be all lowercase. Defaults to lowercased `kind`.",
    )


class CustomResourceSubresourceScale(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    label_selector_path: Optional[str] = Field(
        None,
        alias="labelSelectorPath",
        description="labelSelectorPath defines the JSON path inside of a custom resource that corresponds to Scale `status.selector`. Only JSON paths without the array notation are allowed. Must be a JSON Path under `.status` or `.spec`. Must be set to work with HorizontalPodAutoscaler. The field pointed by this JSON path must be a string field (not a complex selector struct) which contains a serialized label selector in string form. More info: https://kubernetes.io/docs/tasks/access-kubernetes-api/custom-resources/custom-resource-definitions#scale-subresource If there is no value under the given path in the custom resource, the `status.selector` value in the `/scale` subresource will default to the empty string.",
    )
    spec_replicas_path: str = Field(
        ...,
        alias="specReplicasPath",
        description="specReplicasPath defines the JSON path inside of a custom resource that corresponds to Scale `spec.replicas`. Only JSON paths without the array notation are allowed. Must be a JSON Path under `.spec`. If there is no value under the given path in the custom resource, the `/scale` subresource will return an error on GET.",
    )
    status_replicas_path: str = Field(
        ...,
        alias="statusReplicasPath",
        description="statusReplicasPath defines the JSON path inside of a custom resource that corresponds to Scale `status.replicas`. Only JSON paths without the array notation are allowed. Must be a JSON Path under `.status`. If there is no value under the given path in the custom resource, the `status.replicas` value in the `/scale` subresource will default to 0.",
    )


class CustomResourceSubresourceStatus(BaseModel):
    pass

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"


class CustomResourceSubresources(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    scale: Optional[CustomResourceSubresourceScale] = Field(
        None,
        description="scale indicates the custom resource should serve a `/scale` subresource that returns an `autoscaling/v1` Scale object.",
    )


class ExternalDocumentation(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    description: Optional[str] = None
    url: Optional[str] = None


class JSON(BaseModel):
    __root__: Any


class JSONSchemaPropsOrArray(BaseModel):
    __root__: Any


class JSONSchemaPropsOrBool(BaseModel):
    __root__: Any


class JSONSchemaPropsOrStringArray(BaseModel):
    __root__: Any


class ServiceReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: str = Field(..., description="name is the name of the service. Required")
    namespace: str = Field(
        ..., description="namespace is the namespace of the service. Required"
    )
    path: Optional[str] = Field(
        None,
        description="path is an optional URL path at which the webhook will be contacted.",
    )
    port: Optional[int] = Field(
        None,
        description="port is an optional service port at which the webhook will be contacted. `port` should be a valid port number (1-65535, inclusive). Defaults to 443 for backward compatibility.",
    )


class WebhookClientConfig(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ca_bundle: Optional[str] = Field(
        None,
        alias="caBundle",
        description="caBundle is a PEM encoded CA bundle which will be used to validate the webhook's server certificate. If unspecified, system trust roots on the apiserver are used.",
    )
    service: Optional[ServiceReference] = Field(
        None,
        description="service is a reference to the service for this webhook. Either service or url must be specified.\n\nIf the webhook is running within the cluster, then you should use `service`.",
    )
    url: Optional[str] = Field(
        None,
        description='url gives the location of the webhook, in standard URL form (`scheme://host:port/path`). Exactly one of `url` or `service` must be specified.\n\nThe `host` should not refer to a service running in the cluster; use the `service` field instead. The host might be resolved via external DNS in some apiservers (e.g., `kube-apiserver` cannot resolve in-cluster DNS as that would be a layering violation). `host` may also be an IP address.\n\nPlease note that using `localhost` or `127.0.0.1` as a `host` is risky unless you take great care to run this webhook on all hosts which run an apiserver which might need to make calls to this webhook. Such installs are likely to be non-portable, i.e., not easy to turn up in a new cluster.\n\nThe scheme must be "https"; the URL must begin with "https://".\n\nA path is optional, and if present may be any string permissible in a URL. You may use the path to pass an arbitrary string to the webhook, for example, a cluster identifier.\n\nAttempting to use a user or basic auth e.g. "user:password@" is not allowed. Fragments ("#...") and query parameters ("?...") are not allowed, either.',
    )


class CustomResourceConversion(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    conversion_review_versions: Optional[List[str]] = Field(
        None,
        alias="conversionReviewVersions",
        description='conversionReviewVersions is an ordered list of preferred `ConversionReview` versions the Webhook expects. The API server will use the first version in the list which it supports. If none of the versions specified in this list are supported by API server, conversion will fail for the custom resource. If a persisted Webhook configuration specifies allowed versions and does not include any versions known to the API Server, calls to the webhook will fail. Defaults to `["v1beta1"]`.',
    )
    strategy: str = Field(
        ...,
        description="strategy specifies how custom resources are converted between versions. Allowed values are: - `None`: The converter only change the apiVersion and would not touch any other field in the custom resource. - `Webhook`: API Server will call to an external webhook to do the conversion. Additional information\n  is needed for this option. This requires spec.preserveUnknownFields to be false, and spec.conversion.webhookClientConfig to be set.",
    )
    webhook_client_config: Optional[WebhookClientConfig] = Field(
        None,
        alias="webhookClientConfig",
        description="webhookClientConfig is the instructions for how to call the webhook if strategy is `Webhook`. Required when `strategy` is set to `Webhook`.",
    )


class CustomResourceDefinitionCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="lastTransitionTime last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="message is a human-readable message indicating details about last transition.",
    )
    reason: Optional[str] = Field(
        None,
        description="reason is a unique, one-word, CamelCase reason for the condition's last transition.",
    )
    type: str = Field(
        ...,
        description="type is the type of the condition. Types include Established, NamesAccepted and Terminating.",
    )


class CustomResourceDefinitionStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    accepted_names: CustomResourceDefinitionNames = Field(
        ...,
        alias="acceptedNames",
        description="acceptedNames are the names that are actually being used to serve discovery. They may be different than the names in spec.",
    )
    conditions: Optional[List[CustomResourceDefinitionCondition]] = Field(
        None,
        description="conditions indicate state for particular aspects of a CustomResourceDefinition",
    )
    stored_versions: List[str] = Field(
        ...,
        alias="storedVersions",
        description="storedVersions lists all versions of CustomResources that were ever persisted. Tracking these versions allows a migration path for stored versions in etcd. The field is mutable so a migration controller can finish a migration to another version (ensuring no old objects are left in storage), and then remove the rest of the versions from this list. Versions may not be removed from `spec.versions` while they exist in this list.",
    )


class JSONSchemaProps(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    _ref: Optional[str] = Field(None, alias="$ref")
    _schema: Optional[str] = Field(None, alias="$schema")
    additional_items: Optional[JSONSchemaPropsOrBool] = Field(
        None, alias="additionalItems"
    )
    additional_properties: Optional[JSONSchemaPropsOrBool] = Field(
        None, alias="additionalProperties"
    )
    all_of: Optional[List[JSONSchemaProps]] = Field(None, alias="allOf")
    any_of: Optional[List[JSONSchemaProps]] = Field(None, alias="anyOf")
    default: Optional[JSON] = Field(
        None,
        description="default is a default value for undefined object fields. Defaulting is a beta feature under the CustomResourceDefaulting feature gate. CustomResourceDefinitions with defaults must be created using the v1 (or newer) CustomResourceDefinition API.",
    )
    definitions: Optional[Dict[str, Any]] = None
    dependencies: Optional[Dict[str, Any]] = None
    description: Optional[str] = None
    enum: Optional[List[JSON]] = None
    example: Optional[JSON] = None
    exclusive_maximum: Optional[bool] = Field(None, alias="exclusiveMaximum")
    exclusive_minimum: Optional[bool] = Field(None, alias="exclusiveMinimum")
    external_docs: Optional[ExternalDocumentation] = Field(None, alias="externalDocs")
    format: Optional[str] = None
    id: Optional[str] = None
    items: Optional[JSONSchemaPropsOrArray] = None
    max_items: Optional[int] = Field(None, alias="maxItems")
    max_length: Optional[int] = Field(None, alias="maxLength")
    max_properties: Optional[int] = Field(None, alias="maxProperties")
    maximum: Optional[float] = None
    min_items: Optional[int] = Field(None, alias="minItems")
    min_length: Optional[int] = Field(None, alias="minLength")
    min_properties: Optional[int] = Field(None, alias="minProperties")
    minimum: Optional[float] = None
    multiple_of: Optional[float] = Field(None, alias="multipleOf")
    not_: Optional[JSONSchemaProps] = Field(None, alias="not")
    nullable: Optional[bool] = None
    one_of: Optional[List[JSONSchemaProps]] = Field(None, alias="oneOf")
    pattern: Optional[str] = None
    pattern_properties: Optional[Dict[str, Any]] = Field(
        None, alias="patternProperties"
    )
    properties: Optional[Dict[str, Any]] = None
    required: Optional[List[str]] = None
    title: Optional[str] = None
    type: Optional[str] = None
    unique_items: Optional[bool] = Field(None, alias="uniqueItems")
    x_kubernetes_embedded_resource: Optional[bool] = Field(
        None,
        alias="x-kubernetes-embedded-resource",
        description="x-kubernetes-embedded-resource defines that the value is an embedded Kubernetes runtime.Object, with TypeMeta and ObjectMeta. The type must be object. It is allowed to further restrict the embedded object. kind, apiVersion and metadata are validated automatically. x-kubernetes-preserve-unknown-fields is allowed to be true, but does not have to be if the object is fully specified (up to kind, apiVersion, metadata).",
    )
    x_kubernetes_int_or_string: Optional[bool] = Field(
        None,
        alias="x-kubernetes-int-or-string",
        description="x-kubernetes-int-or-string specifies that this value is either an integer or a string. If this is true, an empty type is allowed and type as child of anyOf is permitted if following one of the following patterns:\n\n1) anyOf:\n   - type: integer\n   - type: string\n2) allOf:\n   - anyOf:\n     - type: integer\n     - type: string\n   - ... zero or more",
    )
    x_kubernetes_list_map_keys: Optional[List[str]] = Field(
        None,
        alias="x-kubernetes-list-map-keys",
        description='x-kubernetes-list-map-keys annotates an array with the x-kubernetes-list-type `map` by specifying the keys used as the index of the map.\n\nThis tag MUST only be used on lists that have the "x-kubernetes-list-type" extension set to "map". Also, the values specified for this attribute must be a scalar typed field of the child structure (no nesting is supported).',
    )
    x_kubernetes_list_type: Optional[str] = Field(
        None,
        alias="x-kubernetes-list-type",
        description="x-kubernetes-list-type annotates an array to further describe its topology. This extension must only be used on lists and may have 3 possible values:\n\n1) `atomic`: the list is treated as a single entity, like a scalar.\n     Atomic lists will be entirely replaced when updated. This extension\n     may be used on any type of list (struct, scalar, ...).\n2) `set`:\n     Sets are lists that must not have multiple items with the same value. Each\n     value must be a scalar, an object with x-kubernetes-map-type `atomic` or an\n     array with x-kubernetes-list-type `atomic`.\n3) `map`:\n     These lists are like maps in that their elements have a non-index key\n     used to identify them. Order is preserved upon merge. The map tag\n     must only be used on a list with elements of type object.\nDefaults to atomic for arrays.",
    )
    x_kubernetes_preserve_unknown_fields: Optional[bool] = Field(
        None,
        alias="x-kubernetes-preserve-unknown-fields",
        description="x-kubernetes-preserve-unknown-fields stops the API server decoding step from pruning fields which are not specified in the validation schema. This affects fields recursively, but switches back to normal pruning behaviour if nested properties or additionalProperties are specified in the schema. This can either be true or undefined. False is forbidden.",
    )


class CustomResourceValidation(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    open_apiv3_schema: Optional[JSONSchemaProps] = Field(
        None,
        alias="openAPIV3Schema",
        description="openAPIV3Schema is the OpenAPI v3 schema to use for validation and pruning.",
    )


class CustomResourceDefinitionVersion(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    additional_printer_columns: Optional[List[CustomResourceColumnDefinition]] = Field(
        None,
        alias="additionalPrinterColumns",
        description="additionalPrinterColumns specifies additional columns returned in Table output. See https://kubernetes.io/docs/reference/using-api/api-concepts/#receiving-resources-as-tables for details. Top-level and per-version columns are mutually exclusive. Per-version columns must not all be set to identical values (top-level columns should be used instead). If no top-level or per-version columns are specified, a single column displaying the age of the custom resource is used.",
    )
    name: str = Field(
        ...,
        description="name is the version name, e.g. “v1”, “v2beta1”, etc. The custom resources are served under this version at `/apis/<group>/<version>/...` if `served` is true.",
    )
    schema: Optional[CustomResourceValidation] = Field(
        None,
        description="schema describes the schema used for validation and pruning of this version of the custom resource. Top-level and per-version schemas are mutually exclusive. Per-version schemas must not all be set to identical values (top-level validation schema should be used instead).",
    )
    served: bool = Field(
        ...,
        description="served is a flag enabling/disabling this version from being served via REST APIs",
    )
    storage: bool = Field(
        ...,
        description="storage indicates this version should be used when persisting custom resources to storage. There must be exactly one version with storage=true.",
    )
    subresources: Optional[CustomResourceSubresources] = Field(
        None,
        description="subresources specify what subresources this version of the defined custom resource have. Top-level and per-version subresources are mutually exclusive. Per-version subresources must not all be set to identical values (top-level subresources should be used instead).",
    )


class CustomResourceDefinitionSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    additional_printer_columns: Optional[List[CustomResourceColumnDefinition]] = Field(
        None,
        alias="additionalPrinterColumns",
        description="additionalPrinterColumns specifies additional columns returned in Table output. See https://kubernetes.io/docs/reference/using-api/api-concepts/#receiving-resources-as-tables for details. If present, this field configures columns for all versions. Top-level and per-version columns are mutually exclusive. If no top-level or per-version columns are specified, a single column displaying the age of the custom resource is used.",
    )
    conversion: Optional[CustomResourceConversion] = Field(
        None, description="conversion defines conversion settings for the CRD."
    )
    group: str = Field(
        ...,
        description="group is the API group of the defined custom resource. The custom resources are served under `/apis/<group>/...`. Must match the name of the CustomResourceDefinition (in the form `<names.plural>.<group>`).",
    )
    names: CustomResourceDefinitionNames = Field(
        ...,
        description="names specify the resource and kind names for the custom resource.",
    )
    preserve_unknown_fields: Optional[bool] = Field(
        None,
        alias="preserveUnknownFields",
        description="preserveUnknownFields indicates that object fields which are not specified in the OpenAPI schema should be preserved when persisting to storage. apiVersion, kind, metadata and known fields inside metadata are always preserved. If false, schemas must be defined for all versions. Defaults to true in v1beta for backwards compatibility. Deprecated: will be required to be false in v1. Preservation of unknown fields can be specified in the validation schema using the `x-kubernetes-preserve-unknown-fields: true` extension. See https://kubernetes.io/docs/tasks/access-kubernetes-api/custom-resources/custom-resource-definitions/#pruning-versus-preserving-unknown-fields for details.",
    )
    scope: str = Field(
        ...,
        description="scope indicates whether the defined custom resource is cluster- or namespace-scoped. Allowed values are `Cluster` and `Namespaced`. Default is `Namespaced`.",
    )
    subresources: Optional[CustomResourceSubresources] = Field(
        None,
        description="subresources specify what subresources the defined custom resource has. If present, this field configures subresources for all versions. Top-level and per-version subresources are mutually exclusive.",
    )
    validation: Optional[CustomResourceValidation] = Field(
        None,
        description="validation describes the schema used for validation and pruning of the custom resource. If present, this validation schema is used to validate all versions. Top-level and per-version schemas are mutually exclusive.",
    )
    version: Optional[str] = Field(
        None,
        description="version is the API version of the defined custom resource. The custom resources are served under `/apis/<group>/<version>/...`. Must match the name of the first item in the `versions` list if `version` and `versions` are both specified. Optional if `versions` is specified. Deprecated: use `versions` instead.",
    )
    versions: Optional[List[CustomResourceDefinitionVersion]] = Field(
        None,
        description='versions is the list of all API versions of the defined custom resource. Optional if `version` is specified. The name of the first item in the `versions` list must match the `version` field if `version` and `versions` are both specified. Version names are used to compute the order in which served versions are listed in API discovery. If the version string is "kube-like", it will sort above non "kube-like" version strings, which are ordered lexicographically. "Kube-like" versions start with a "v", then are followed by a number (the major version), then optionally the string "alpha" or "beta" and another number (the minor version). These are sorted first by GA > beta > alpha (where GA is a version with no suffix such as beta or alpha), and then by comparing major version, then minor version. An example sorted list of versions: v10, v2, v1, v11beta2, v10beta3, v3beta1, v12alpha1, v11alpha2, foo1, foo10.',
    )


class CustomResourceDefinition(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apiextensions.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind181] = Field(
        "CustomResourceDefinition",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: CustomResourceDefinitionSpec = Field(
        ..., description="spec describes how the user wants the resources to appear"
    )


class CustomResourceDefinitionList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apiextensions.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[CustomResourceDefinition] = Field(
        ..., description="items list individual CustomResourceDefinition objects"
    )
    kind: Optional[Kind182] = Field(
        "CustomResourceDefinitionList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = None


io.k8s.apiextensions_apiserver.pkg.apis.apiextensions.v1beta1.JSONSchemaProps.update_forward_refs()



#!/usr/bin/env python
"""
Chart Description
"""
import pdk8s
import pdk8s.chart_cli
from pdk8s import k8s


# Name of the chart
name = "{{ cookiecutter.chart_name }}"

# This is the chart version. This version number should be incremented each time you make changes
# to the chart and its templates, including the app version.
# Versions are expected to follow Semantic Versioning (https://semver.org/)
chart_version = "{{ cookiecutter.chart_name }}"

# This is the version number of the application being deployed. This version number should be
# incremented each time you make changes to the application. Versions are not expected to
# follow Semantic Versioning. They should reflect the version the application is using.
app_version = "{{ cookiecutter.chart_name }}"

label = {"app": "hello-k8s"}

# this is your chart
chart = [
    k8s.Deployment(name='deployment',
                    spec=k8s.DeploymentSpec(
                        replicas=2,
                        selector=k8s.LabelSelector(match_labels=label),
                        template=k8s.PodTemplateSpec(
                        metadata=k8s.ObjectMeta(labels=label),
                        spec=k8s.PodSpec(containers=[
                            k8s.Container(
                            name='hello-kubernetes',
                            image='paulbouwer/hello-kubernetes:1.7',
                            ports=[k8s.ContainerPort(container_port=8080)])]))))
]


if __name__ == "__main__":
    pdk8s.chart_cli.main(obj=locals())


"""Console script for pdk8s."""
import os
import sys
import collections.abc

import yaml
import click

import pdk8s.synth
from . import k8s


option_input = click.option("-i", "--input", "input_path", default="chart.py", type=click.Path(exists=True, dir_okay=False), help="Python file that generates your chart")
option_output = click.option("-o", "--output", "output_path", default="dist", type=click.Path(file_okay=False), help="Folder in which to store generated chart")
option_format = click.option("--format", "output_format", default="helm", type=click.Choice(['helm', 'kubernetes'], case_sensitive=False), help="Format of output")


@click.group()
def main():
    """Console script for pdk8s."""


def load_chart_from_file(path):
    chart_variables = {}
    exec(open(path).read(), {}, chart_variables)
    return chart_variables


def add_subcommands(main, with_input):
    @main.command()

    @option_output
    @option_format
    @click.pass_context
    def synth(ctx, output_path, output_format, input_path=None):
        if input_path:
            chart_variables = load_chart_from_file(input_path)
        else:
            chart_variables = ctx.obj

        return pdk8s.synth.synth(chart_variables, output_format, output_path)

    if with_input:
        option_input(synth)

    @main.command()
    def init():
        import pkg_resources
        import cookiecutter.main

        path = pkg_resources.resource_filename("pdk8s", "cookiecutter")
        cookiecutter.main.cookiecutter(path)

    @main.command()
    def watch():
        print("not yet implemented")
        return 99


add_subcommands(main, with_input=True)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional, Union

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind122, Kind123
from ...apimachinery.pkg.apis.meta import v1
from ...apimachinery.pkg.util import intstr
from ..core import v1


class IngressStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    load_balancer: Optional[v1_1.LoadBalancerStatus] = Field(
        None,
        alias="loadBalancer",
        description="LoadBalancer contains the current status of the load-balancer.",
    )


class IngressTLS(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    hosts: Optional[List[str]] = Field(
        None,
        description="Hosts are a list of hosts included in the TLS certificate. The values in this list must match the name/s used in the tlsSecret. Defaults to the wildcard host setting for the loadbalancer controller fulfilling this Ingress, if left unspecified.",
    )
    secret_name: Optional[str] = Field(
        None,
        alias="secretName",
        description='SecretName is the name of the secret used to terminate SSL traffic on 443. Field is left optional to allow SSL routing based on SNI hostname alone. If the SNI host in a listener conflicts with the "Host" header field used by an IngressRule, the SNI host is used for termination and value of the Host header is used for routing.',
    )


class IngressBackend(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    service_name: str = Field(
        ...,
        alias="serviceName",
        description="Specifies the name of the referenced service.",
    )
    service_port: Union[int, str] = Field(
        ...,
        alias="servicePort",
        description="Specifies the port of the referenced service.",
    )


class HTTPIngressPath(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    backend: IngressBackend = Field(
        ...,
        description="Backend defines the referenced service endpoint to which the traffic will be forwarded to.",
    )
    path: Optional[str] = Field(
        None,
        description="Path is an extended POSIX regex as defined by IEEE Std 1003.1, (i.e this follows the egrep/unix syntax, not the perl syntax) matched against the path of an incoming request. Currently it can contain characters disallowed from the conventional \"path\" part of a URL as defined by RFC 3986. Paths must begin with a '/'. If unspecified, the path defaults to a catch all sending traffic to the backend.",
    )


class HTTPIngressRuleValue(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    paths: List[HTTPIngressPath] = Field(
        ..., description="A collection of paths that map requests to backends."
    )


class IngressRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    host: Optional[str] = Field(
        None,
        description='Host is the fully qualified domain name of a network host, as defined by RFC 3986. Note the following deviations from the "host" part of the URI as defined in the RFC: 1. IPs are not allowed. Currently an IngressRuleValue can only apply to the\n\t  IP in the Spec of the parent Ingress.\n2. The `:` delimiter is not respected because ports are not allowed.\n\t  Currently the port of an Ingress is implicitly :80 for http and\n\t  :443 for https.\nBoth these may change in the future. Incoming requests are matched against the host before the IngressRuleValue. If the host is unspecified, the Ingress routes all traffic based on the specified IngressRuleValue.',
    )
    http: Optional[HTTPIngressRuleValue] = None


class IngressSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    backend: Optional[IngressBackend] = Field(
        None,
        description="A default backend capable of servicing requests that don't match any rule. At least one of 'backend' or 'rules' must be specified. This field is optional to allow the loadbalancer controller or defaulting logic to specify a global default.",
    )
    rules: Optional[List[IngressRule]] = Field(
        None,
        description="A list of host rules used to configure the Ingress. If unspecified, or no rule matches, all traffic is sent to the default backend.",
    )
    tls: Optional[List[IngressTLS]] = Field(
        None,
        description="TLS configuration. Currently the Ingress only supports a single TLS port, 443. If multiple members of this list specify different hosts, they will be multiplexed on the same port according to the hostname specified through the SNI TLS extension, if the ingress controller fulfilling the ingress supports SNI.",
    )


class Ingress(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "networking.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind122] = Field(
        "Ingress",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[IngressSpec] = Field(
        None,
        description="Spec is the desired state of the Ingress. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class IngressList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "networking.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Ingress] = Field(..., description="Items is the list of Ingress.")
    kind: Optional[Kind123] = Field(
        "IngressList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



import subprocess

import yaml

class Helm:
    def __init__(self, namespace=None):
        self.bin = "helm"
        self.namespace = namespace
    
    def get(self, resource_type, *args):
        pass

    def exec(self, *args):
        cmd = [self.bin] + list(args) + ["-o", "yaml"]
        if self.namespace:
            cmd += ['-n', self.namespace]
        
        data = subprocess.check_output(cmd)
        return yaml.safe_load(data)

    def install(self):
        pass

    def upgrade(self):
        pass


helm = Helm()

upgrade = helm.upgrade
install = helm.install


# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind133, Kind134, Kind135, Kind136, Kind137, Kind138, Kind139, Kind140
from ...apimachinery.pkg.apis.meta import v1


class PolicyRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_groups: Optional[List[str]] = Field(
        None,
        alias="apiGroups",
        description="APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.",
    )
    non_resource_ur_ls: Optional[List[str]] = Field(
        None,
        alias="nonResourceURLs",
        description='NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as "pods" or "secrets") or non-resource URL paths (such as "/api"),  but not both.',
    )
    resource_names: Optional[List[str]] = Field(
        None,
        alias="resourceNames",
        description="ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.",
    )
    resources: Optional[List[str]] = Field(
        None,
        description="Resources is a list of resources this rule applies to.  ResourceAll represents all resources.",
    )
    verbs: List[str] = Field(
        ...,
        description="Verbs is a list of Verbs that apply to ALL the ResourceKinds and AttributeRestrictions contained in this rule.  VerbAll represents all kinds.",
    )


class RoleRef(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_group: str = Field(
        ...,
        alias="apiGroup",
        description="APIGroup is the group for the resource being referenced",
    )
    kind: str = Field(..., description="Kind is the type of resource being referenced")
    name: str = Field(..., description="Name is the name of resource being referenced")


class Subject(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_group: Optional[str] = Field(
        None,
        alias="apiGroup",
        description='APIGroup holds the API group of the referenced subject. Defaults to "" for ServiceAccount subjects. Defaults to "rbac.authorization.k8s.io" for User and Group subjects.',
    )
    kind: str = Field(
        ...,
        description='Kind of object being referenced. Values defined by this API group are "User", "Group", and "ServiceAccount". If the Authorizer does not recognized the kind value, the Authorizer should report an error.',
    )
    name: str = Field(..., description="Name of the object being referenced.")
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referenced object.  If the object kind is non-namespace, such as "User" or "Group", and this value is not empty the Authorizer should report an error.',
    )


class AggregationRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    cluster_role_selectors: Optional[List[v1.LabelSelector]] = Field(
        None,
        alias="clusterRoleSelectors",
        description="ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added",
    )


class ClusterRole(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    aggregation_rule: Optional[AggregationRule] = Field(
        None,
        alias="aggregationRule",
        description="AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.",
    )
    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind133] = Field(
        "ClusterRole",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object's metadata."
    )
    rules: Optional[List[PolicyRule]] = Field(
        None, description="Rules holds all the PolicyRules for this ClusterRole"
    )


class ClusterRoleBinding(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind134] = Field(
        "ClusterRoleBinding",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object's metadata."
    )
    role_ref: RoleRef = Field(
        ...,
        alias="roleRef",
        description="RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
    )
    subjects: Optional[List[Subject]] = Field(
        None,
        description="Subjects holds references to the objects the role applies to.",
    )


class ClusterRoleBindingList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[ClusterRoleBinding] = Field(
        ..., description="Items is a list of ClusterRoleBindings"
    )
    kind: Optional[Kind135] = Field(
        "ClusterRoleBindingList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None, description="Standard object's metadata."
    )


class ClusterRoleList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[ClusterRole] = Field(..., description="Items is a list of ClusterRoles")
    kind: Optional[Kind136] = Field(
        "ClusterRoleList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None, description="Standard object's metadata."
    )


class Role(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind137] = Field(
        "Role",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object's metadata."
    )
    rules: Optional[List[PolicyRule]] = Field(
        None, description="Rules holds all the PolicyRules for this Role"
    )


class RoleBinding(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind138] = Field(
        "RoleBinding",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object's metadata."
    )
    role_ref: RoleRef = Field(
        ...,
        alias="roleRef",
        description="RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error.",
    )
    subjects: Optional[List[Subject]] = Field(
        None,
        description="Subjects holds references to the objects the role applies to.",
    )


class RoleBindingList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[RoleBinding] = Field(..., description="Items is a list of RoleBindings")
    kind: Optional[Kind139] = Field(
        "RoleBindingList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None, description="Standard object's metadata."
    )


class RoleList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "rbac.authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Role] = Field(..., description="Items is a list of Roles")
    kind: Optional[Kind140] = Field(
        "RoleList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None, description="Standard object's metadata."
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind65, Kind66
from ...apimachinery.pkg.apis.meta import v1


class LeaseSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    acquire_time: Optional[v1.MicroTime] = Field(
        None,
        alias="acquireTime",
        description="acquireTime is a time when the current lease was acquired.",
    )
    holder_identity: Optional[str] = Field(
        None,
        alias="holderIdentity",
        description="holderIdentity contains the identity of the holder of a current lease.",
    )
    lease_duration_seconds: Optional[int] = Field(
        None,
        alias="leaseDurationSeconds",
        description="leaseDurationSeconds is a duration that candidates for a lease need to wait to force acquire it. This is measure against time of last observed RenewTime.",
    )
    lease_transitions: Optional[int] = Field(
        None,
        alias="leaseTransitions",
        description="leaseTransitions is the number of transitions of a lease between holders.",
    )
    renew_time: Optional[v1.MicroTime] = Field(
        None,
        alias="renewTime",
        description="renewTime is a time when the current holder of a lease has last updated the lease.",
    )


class Lease(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "coordination.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind65] = Field(
        "Lease",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[LeaseSpec] = Field(
        None,
        description="Specification of the Lease. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class LeaseList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "coordination.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Lease] = Field(..., description="Items is a list of schema objects.")
    kind: Optional[Kind66] = Field(
        "LeaseList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
